@@ -55,6 +55,7 @@ class DistributionFiles:
     #  - allowempty == True
     #  - be listed in SEARCH_STRING
     #  - have a function get_distribution_DISTNAME implemented
+    # keep names in sync with Conditionals page of docs
     OSDIST_LIST = (
         {'path': '/etc/oracle-release', 'name': 'OracleLinux'},
         {'path': '/etc/slackware-version', 'name': 'Slackware'},
@@ -65,10 +66,11 @@ class DistributionFiles:
         {'path': '/etc/alpine-release', 'name': 'Alpine'},
         {'path': '/etc/arch-release', 'name': 'Archlinux', 'allowempty': True},
         {'path': '/etc/os-release', 'name': 'Archlinux'},
-        {'path': '/etc/os-release', 'name': 'SuSE'},
-        {'path': '/etc/SuSE-release', 'name': 'SuSE'},
+        {'path': '/etc/os-release', 'name': 'SUSE'},
+        {'path': '/etc/SuSE-release', 'name': 'SUSE'},
         {'path': '/etc/gentoo-release', 'name': 'Gentoo'},
         {'path': '/etc/os-release', 'name': 'Debian'},
+        {'path': '/etc/lsb-release', 'name': 'Debian'},
         {'path': '/etc/lsb-release', 'name': 'Mandriva'},
         {'path': '/etc/altlinux-release', 'name': 'Altlinux'},
         {'path': '/etc/sourcemage-release', 'name': 'SMGL'},
@@ -81,7 +83,7 @@ class DistributionFiles:
         'OracleLinux': 'Oracle Linux',
         'RedHat': 'Red Hat',
         'Altlinux': 'ALT Linux',
-        'ClearLinux': 'Clear Linux Software for Intel Architecture',
+        'ClearLinux': 'Clear Linux',
         'SMGL': 'Source Mage GNU/Linux',
     }
 
@@ -113,11 +115,10 @@ class DistributionFiles:
             # only the distribution name is set, the version is assumed to be correct from platform.dist()
             if self.SEARCH_STRING[name] in dist_file_content:
                 # this sets distribution=RedHat if 'Red Hat' shows up in data
-                # self.facts['distribution'] = name
                 dist_file_dict['distribution'] = name
+                dist_file_dict['distribution_file_search_string'] = self.SEARCH_STRING[name]
             else:
                 # this sets distribution to what's in the data, e.g. CentOS, Scientific, ...
-                # self.facts['distribution'] = dist_file_content.split()[0]
                 dist_file_dict['distribution'] = dist_file_content.split()[0]
 
             return True, dist_file_dict
@@ -126,15 +127,14 @@ class DistributionFiles:
             if self.OS_RELEASE_ALIAS[name] in dist_file_content:
                 dist_file_dict['distribution'] = name
                 return True, dist_file_dict
+            return False, dist_file_dict
 
         # call a dedicated function for parsing the file content
         # TODO: replace with a map or a class
         try:
             # FIXME: most of these dont actually look at the dist file contents, but random other stuff
             distfunc_name = 'parse_distribution_file_' + name
-            # print('distfunc_name: %s' % distfunc_name)
             distfunc = getattr(self, distfunc_name)
-            # print('distfunc: %s' % distfunc)
             parsed, dist_file_dict = distfunc(name, dist_file_content, path, collected_facts)
             return parsed, dist_file_dict
         except AttributeError as exc:
@@ -180,31 +180,29 @@ class DistributionFiles:
             # /etc/os-release with a different name
             if has_dist_file and allow_empty:
                 dist_file_facts['distribution'] = name
+                dist_file_facts['distribution_file_path'] = path
+                dist_file_facts['distribution_file_variety'] = name
                 break
 
             if not has_dist_file:
                 # keep looking
                 continue
 
-            # first valid os dist file we find we count
-            # FIXME: coreos and a few other bits expect this
-            # self.facts['distribution'] = name
-            dist_file_facts['distribution'] = name
-            dist_file_facts['distribution_file_variety'] = name
-            dist_file_facts['distribution_file_path'] = path
-
             parsed_dist_file, parsed_dist_file_facts = self._parse_dist_file(name, dist_file_content, path, dist_file_facts)
 
-            dist_file_facts['distribution_file_parsed'] = parsed_dist_file
-
-            # finally found the right os dit file and were able to parse it
+            # finally found the right os dist file and were able to parse it
             if parsed_dist_file:
+                dist_file_facts['distribution'] = name
+                dist_file_facts['distribution_file_path'] = path
+                # distribution and file_variety are the same here, but distribution
+                # will be changed/mapped to a more specific name.
+                # ie, dist=Fedora, file_variety=RedHat
+                dist_file_facts['distribution_file_variety'] = name
+                dist_file_facts['distribution_file_parsed'] = parsed_dist_file
                 dist_file_facts.update(parsed_dist_file_facts)
                 break
 
         return dist_file_facts
-#        distribution_facts.update(dist_file_facts)
-#        return distribution_facts
 
     # TODO: FIXME: split distro file parsing into its own module or class
     def parse_distribution_file_Slackware(self, name, data, path, collected_facts):
@@ -212,7 +210,7 @@ class DistributionFiles:
         if 'Slackware' not in data:
             return False, slackware_facts  # TODO: remove
         slackware_facts['distribution'] = name
-        version = re.findall('\w+[.]\w+', data)
+        version = re.findall(r'\w+[.]\w+', data)
         if version:
             slackware_facts['distribution_version'] = version[0]
         return True, slackware_facts
@@ -245,7 +243,7 @@ class DistributionFiles:
         alpine_facts['distribution_version'] = data
         return True, alpine_facts
 
-    def parse_distribution_file_SuSE(self, name, data, path, collected_facts):
+    def parse_distribution_file_SUSE(self, name, data, path, collected_facts):
         suse_facts = {}
         if 'suse' not in data.lower():
             return False, suse_facts  # TODO: remove if tested without this
@@ -255,16 +253,17 @@ class DistributionFiles:
                 if distribution:
                     suse_facts['distribution'] = distribution.group(1).strip('"')
                 # example pattern are 13.04 13.0 13
-                distribution_version = re.search('^VERSION_ID="?([0-9]+\.?[0-9]*)"?', line)
+                distribution_version = re.search(r'^VERSION_ID="?([0-9]+\.?[0-9]*)"?', line)
                 if distribution_version:
                     suse_facts['distribution_version'] = distribution_version.group(1)
+                    suse_facts['distribution_major_version'] = distribution_version.group(1).split('.')[0]
                 if 'open' in data.lower():
-                    release = re.search('^VERSION_ID="?[0-9]+\.?([0-9]*)"?', line)
+                    release = re.search(r'^VERSION_ID="?[0-9]+\.?([0-9]*)"?', line)
                     if release:
                         suse_facts['distribution_release'] = release.groups()[0]
                 elif 'enterprise' in data.lower() and 'VERSION_ID' in line:
                     # SLES doesn't got funny release names
-                    release = re.search('^VERSION_ID="?[0-9]+\.?([0-9]*)"?', line)
+                    release = re.search(r'^VERSION_ID="?[0-9]+\.?([0-9]*)"?', line)
                     if release.group(1):
                         release = release.group(1)
                     else:
@@ -298,7 +297,7 @@ class DistributionFiles:
         debian_facts = {}
         if 'Debian' in data or 'Raspbian' in data:
             debian_facts['distribution'] = 'Debian'
-            release = re.search("PRETTY_NAME=[^(]+ \(?([^)]+?)\)", data)
+            release = re.search(r"PRETTY_NAME=[^(]+ \(?([^)]+?)\)", data)
             if release:
                 debian_facts['distribution_release'] = release.groups()[0]
 
@@ -313,6 +312,9 @@ class DistributionFiles:
         elif 'Ubuntu' in data:
             debian_facts['distribution'] = 'Ubuntu'
             # nothing else to do, Ubuntu gets correct info from python functions
+        elif 'SteamOS' in data:
+            debian_facts['distribution'] = 'SteamOS'
+            # nothing else to do, SteamOS gets correct info from python functions
         else:
             return False, debian_facts
 
@@ -338,7 +340,7 @@ class DistributionFiles:
         na_facts = {}
         for line in data.splitlines():
             distribution = re.search("^NAME=(.*)", line)
-            if distribution and collected_facts['distribution'] == 'NA':
+            if distribution and name == 'NA':
                 na_facts['distribution'] = distribution.group(1).strip('"')
             version = re.search("^VERSION=(.*)", line)
             if version and collected_facts['distribution_version'] == 'NA':
@@ -387,8 +389,8 @@ class Distribution(object):
         {'path': '/etc/system-release', 'name': 'Amazon'},
         {'path': '/etc/alpine-release', 'name': 'Alpine'},
         {'path': '/etc/arch-release', 'name': 'Archlinux', 'allowempty': True},
-        {'path': '/etc/os-release', 'name': 'SuSE'},
-        {'path': '/etc/SuSE-release', 'name': 'SuSE'},
+        {'path': '/etc/os-release', 'name': 'SUSE'},
+        {'path': '/etc/SuSE-release', 'name': 'SUSE'},
         {'path': '/etc/gentoo-release', 'name': 'Gentoo'},
         {'path': '/etc/os-release', 'name': 'Debian'},
         {'path': '/etc/lsb-release', 'name': 'Mandriva'},
@@ -407,11 +409,12 @@ class Distribution(object):
         'SMGL': 'Source Mage GNU/Linux',
     }
 
+    # keep keys in sync with Conditionals page of docs
     OS_FAMILY_MAP = {'RedHat': ['RedHat', 'Fedora', 'CentOS', 'Scientific', 'SLC',
                                 'Ascendos', 'CloudLinux', 'PSBM', 'OracleLinux', 'OVS',
                                 'OEL', 'Amazon', 'Virtuozzo', 'XenServer'],
                      'Debian': ['Debian', 'Ubuntu', 'Raspbian', 'Neon', 'KDE neon',
-                                'Linux Mint'],
+                                'Linux Mint', 'SteamOS'],
                      'Suse': ['SuSE', 'SLES', 'SLED', 'openSUSE', 'openSUSE Tumbleweed',
                               'SLES_SAP', 'SUSE_LINUX', 'openSUSE Leap'],
                      'Archlinux': ['Archlinux', 'Antergos', 'Manjaro'],
@@ -425,7 +428,7 @@ class Distribution(object):
                      'AIX': ['AIX'],
                      'HP-UX': ['HPUX'],
                      'Darwin': ['MacOSX'],
-                     'FreeBSD': ['FreeBSD']}
+                     'FreeBSD': ['FreeBSD', 'TrueOS']}
 
     OS_FAMILY = {}
     for family, names in OS_FAMILY_MAP.items():
@@ -473,14 +476,15 @@ class Distribution(object):
         aix_facts = {}
         rc, out, err = self.module.run_command("/usr/bin/oslevel")
         data = out.split('.')
+        aix_facts['distribution_major_version'] = data[0]
         aix_facts['distribution_version'] = data[0]
         aix_facts['distribution_release'] = data[1]
         return aix_facts
 
     def get_distribution_HPUX(self):
         hpux_facts = {}
-        rc, out, err = self.module.run_command("/usr/sbin/swlist |egrep 'HPUX.*OE.*[AB].[0-9]+\.[0-9]+'", use_unsafe_shell=True)
-        data = re.search('HPUX.*OE.*([AB].[0-9]+\.[0-9]+)\.([0-9]+).*', out)
+        rc, out, err = self.module.run_command(r"/usr/sbin/swlist |egrep 'HPUX.*OE.*[AB].[0-9]+\.[0-9]+'", use_unsafe_shell=True)
+        data = re.search(r'HPUX.*OE.*([AB].[0-9]+\.[0-9]+)\.([0-9]+).*', out)
         if data:
             hpux_facts['distribution_version'] = data.groups()[0]
             hpux_facts['distribution_release'] = data.groups()[1]
@@ -499,7 +503,9 @@ class Distribution(object):
     def get_distribution_FreeBSD(self):
         freebsd_facts = {}
         freebsd_facts['distribution_release'] = platform.release()
-        data = re.search('(\d+)\.(\d+)-RELEASE.*', freebsd_facts['distribution_release'])
+        data = re.search(r'(\d+)\.(\d+)-(RELEASE|STABLE|CURRENT).*', freebsd_facts['distribution_release'])
+        if 'trueos' in platform.version():
+            freebsd_facts['distribution'] = 'TrueOS'
         if data:
             freebsd_facts['distribution_major_version'] = data.group(1)
             freebsd_facts['distribution_version'] = '%s.%s' % (data.group(1), data.group(2))
@@ -509,7 +515,7 @@ class Distribution(object):
         openbsd_facts = {}
         openbsd_facts['distribution_version'] = platform.release()
         rc, out, err = self.module.run_command("/sbin/sysctl -n kern.version")
-        match = re.match('OpenBSD\s[0-9]+.[0-9]+-(\S+)\s.*', out)
+        match = re.match(r'OpenBSD\s[0-9]+.[0-9]+-(\S+)\s.*', out)
         if match:
             openbsd_facts['distribution_release'] = match.groups()[0]
         else:
@@ -534,11 +540,8 @@ class Distribution(object):
     def get_distribution_SunOS(self):
         sunos_facts = {}
 
-        # print('platform.release: %s' % distribution_release)
         data = get_file_content('/etc/release').splitlines()[0]
 
-        # print('get_file_content: data=%s' % data)
-
         if 'Solaris' in data:
             ora_prefix = ''
             if 'Oracle Solaris' in data:
@@ -552,8 +555,6 @@ class Distribution(object):
         uname_v = get_uname_version(self.module)
         distribution_version = None
 
-        # print('uname_v: %s' % uname_v)
-
         if 'SmartOS' in data:
             sunos_facts['distribution'] = 'SmartOS'
             if _file_exists('/etc/product'):
@@ -569,7 +570,6 @@ class Distribution(object):
             sunos_facts['distribution'] = 'Nexenta'
             distribution_version = data.split()[-1].lstrip('v')
 
-        # print('sunos_facts: %s' % sunos_facts)
         if sunos_facts.get('distribution', '') in ('SmartOS', 'OpenIndiana', 'OmniOS', 'Nexenta'):
             sunos_facts['distribution_release'] = data.strip()
             if distribution_version is not None:
@@ -585,7 +585,8 @@ class DistributionFactCollector(BaseFactCollector):
     name = 'distribution'
     _fact_ids = set(['distribution_version',
                      'distribution_release',
-                     'distribution_major_version'])
+                     'distribution_major_version',
+                     'os_family'])
 
     def collect(self, module=None, collected_facts=None):
         collected_facts = collected_facts or {}
