@@ -28,7 +28,6 @@ options:
   state:
     description:
       - Create or delete the table
-    required: false
     choices: ['present', 'absent']
     default: 'present'
   name:
@@ -39,34 +38,26 @@ options:
     description:
       - Name of the hash key.
       - Required when C(state=present).
-    required: false
-    default: null
   hash_key_type:
     description:
       - Type of the hash key.
-    required: false
     choices: ['STRING', 'NUMBER', 'BINARY']
     default: 'STRING'
   range_key_name:
     description:
       - Name of the range key.
-    required: false
-    default: null
   range_key_type:
     description:
       - Type of the range key.
-    required: false
     choices: ['STRING', 'NUMBER', 'BINARY']
     default: 'STRING'
   read_capacity:
     description:
       - Read throughput capacity (units) to provision.
-    required: false
     default: 1
   write_capacity:
     description:
       - Write throughput capacity (units) to provision.
-    required: false
     default: 1
   indexes:
     description:
@@ -74,20 +65,16 @@ options:
       - "required options: ['name', 'type', 'hash_key_name']"
       - "valid types: ['all', 'global_all', 'global_include', 'global_keys_only', 'include', 'keys_only']"
       - "other options: ['hash_key_type', 'range_key_name', 'range_key_type', 'includes', 'read_capacity', 'write_capacity']"
-    required: false
     default: []
     version_added: "2.1"
   tags:
     version_added: "2.4"
     description:
       - a hash/dictionary of tags to add to the new instance or for starting/stopping instance by tag; '{"key":"value"}' and '{"key":"value","key":"value"}'
-    required: false
-    default: null
   wait_for_active_timeout:
     version_added: "2.4"
     description:
       - how long before wait gives up, in seconds. only used when tags is set
-    required: false
     default: 60
 extends_documentation_fragment:
     - aws
@@ -223,7 +210,6 @@ def create_or_update_dynamo_table(connection, module, boto3_dynamodb=None, boto3
     try:
         table = Table(table_name, connection=connection)
 
-
         if dynamo_table_exists(table):
             result['changed'] = update_dynamo_table(table, throughput=throughput, check_mode=module.check_mode, global_indexes=global_indexes)
         else:
@@ -349,7 +335,7 @@ def has_throughput_changed(table, new_throughput):
         return False
 
     return new_throughput['read'] != table.throughput['read'] or \
-           new_throughput['write'] != table.throughput['write']
+        new_throughput['write'] != table.throughput['write']
 
 
 def get_schema_param(hash_key_name, hash_key_type, range_key_name, range_key_type):
@@ -397,6 +383,7 @@ def validate_index(index, module):
     if index['type'] not in INDEX_TYPE_OPTIONS:
         module.fail_json(msg='%s is not a valid index type, must be one of %s' % (index['type'], INDEX_TYPE_OPTIONS))
 
+
 def get_indexes(all_indexes):
     indexes = []
     global_indexes = []
@@ -429,7 +416,6 @@ def get_indexes(all_indexes):
     return indexes, global_indexes
 
 
-
 def main():
     argument_spec = ec2_argument_spec()
     argument_spec.update(dict(
@@ -442,8 +428,8 @@ def main():
         read_capacity=dict(default=1, type='int'),
         write_capacity=dict(default=1, type='int'),
         indexes=dict(default=[], type='list'),
-        tags = dict(type='dict'),
-        wait_for_active_timeout = dict(default=60, type='int'),
+        tags=dict(type='dict'),
+        wait_for_active_timeout=dict(default=60, type='int'),
     ))
 
     module = AnsibleModule(
