@@ -1,22 +1,15 @@
 #!/usr/bin/python
 #
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+# Copyright (c) Ansible Project
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
+
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
-                    'supported_by': 'curated'}
+                    'supported_by': 'certified'}
 
 
 DOCUMENTATION = '''
@@ -37,16 +30,16 @@ options:
     description:
       - The resource group name to use or create to host the deployed template
     required: true
+    aliases:
+      - resource_group
   location:
     description:
       - The geo-locations in which the resource group will be located.
-    required: false
     default: westus
   deployment_mode:
     description:
       - In incremental mode, resources are deployed without deleting existing resources that are not included in the template.
         In complete mode resources are deployed and existing resources in the resource group not included in the template are deleted.
-    required: false
     default: incremental
     choices:
         - complete
@@ -56,7 +49,6 @@ options:
       - If state is "present", template will be created. If state is "present" and if deployment exists, it will be
         updated. If state is "absent", stack will be removed.
     default: present
-    required: false
     choices:
         - present
         - absent
@@ -64,26 +56,20 @@ options:
     description:
       - A hash containing the templates inline. This parameter is mutually exclusive with 'template_link'.
         Either one of them is required if "state" parameter is "present".
-    required: false
-    default: null
+    type: dict
   template_link:
     description:
       - Uri of file containing the template body. This parameter is mutually exclusive with 'template'. Either one
         of them is required if "state" parameter is "present".
-    required: false
-    default: null
   parameters:
     description:
       - A hash of all the required template variables for the deployment template. This parameter is mutually exclusive
         with 'parameters_link'. Either one of them is required if "state" parameter is "present".
-    required: false
-    default: null
+    type: dict
   parameters_link:
     description:
       - Uri of file containing the parameters body. This parameter is mutually exclusive with 'parameters'. Either
         one of them is required if "state" parameter is "present".
-    required: false
-    default: null
   deployment_name:
     description:
       - The name of the deployment to be tracked in the resource group deployment history. Re-using a deployment name
@@ -92,8 +78,8 @@ options:
   wait_for_deployment_completion:
     description:
       - Whether or not to block until the deployment has completed.
-    default: yes
-    choices: ['yes', 'no']
+    type: bool
+    default: 'yes'
   wait_for_deployment_polling_period:
     description:
       - Time (in seconds) to wait between polls when waiting for deployment completion.
@@ -101,6 +87,7 @@ options:
 
 extends_documentation_fragment:
     - azure
+    - azure_tags
 
 author:
     - David Justice (@devigned)
@@ -233,7 +220,9 @@ EXAMPLES = '''
             - "14.04.2-LTS"
             - "15.04"
           metadata:
-            description: "The Ubuntu version for the VM. This will pick a fully patched image of this given Ubuntu version. Allowed values: 12.04.5-LTS, 14.04.2-LTS, 15.04."
+            description: >
+                         The Ubuntu version for the VM. This will pick a fully patched image of this given Ubuntu version.
+                         Allowed values: 12.04.5-LTS, 14.04.2-LTS, 15.04."
       variables:
         location: "West US"
         imagePublisher: "Canonical"
@@ -320,7 +309,9 @@ EXAMPLES = '''
               osDisk:
                 name: "osdisk"
                 vhd:
-                  uri: "[concat('http://',parameters('newStorageAccountName'),'.blob.core.windows.net/',variables('vmStorageAccountContainerName'),'/',variables('OSDiskName'),'.vhd')]"
+                  uri: >
+                       [concat('http://',parameters('newStorageAccountName'),'.blob.core.windows.net/',variables('vmStorageAccountContainerName'),'/',
+                       variables('OSDiskName'),'.vhd')]
                 caching: "ReadWrite"
                 createOption: "FromImage"
             networkProfile:
@@ -370,26 +361,18 @@ deployment:
         returned: always
 '''
 
-PREREQ_IMPORT_ERROR = None
+import time
 
 try:
+    from azure.common.credentials import ServicePrincipalCredentials
     import time
     import yaml
 except ImportError as exc:
     IMPORT_ERROR = "Error importing module prerequisites: %s" % exc
 
-from ansible.module_utils.azure_rm_common import *
-
 try:
     from itertools import chain
-    from azure.common.credentials import ServicePrincipalCredentials
     from azure.common.exceptions import CloudError
-    from azure.mgmt.resource.resources.models import (DeploymentProperties,
-                                                      ParametersLink,
-                                                      TemplateLink,
-                                                      Deployment,
-                                                      ResourceGroup,
-                                                      Dependency)
     from azure.mgmt.resource.resources import ResourceManagementClient
     from azure.mgmt.network import NetworkManagementClient
 
@@ -397,6 +380,8 @@ except ImportError:
     # This is handled in azure_rm_common
     pass
 
+from ansible.module_utils.azure_rm_common import AzureRMModuleBase
+
 
 class AzureRMDeploymentManager(AzureRMModuleBase):
 
@@ -431,6 +416,7 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
         self.wait_for_deployment_completion = None
         self.wait_for_deployment_polling_period = None
         self.tags = None
+        self.append_tags = None
 
         self.results = dict(
             deployment=dict(),
@@ -444,28 +430,39 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
 
     def exec_module(self, **kwargs):
 
-        if PREREQ_IMPORT_ERROR:
-            self.fail(PREREQ_IMPORT_ERROR)
-
-        for key in self.module_arg_spec.keys() + ['tags']:
+        for key in list(self.module_arg_spec.keys()) + ['append_tags', 'tags']:
             setattr(self, key, kwargs[key])
 
         if self.state == 'present':
             deployment = self.deploy_template()
-            self.results['deployment'] = dict(
-                name=deployment.name,
-                group_name=self.resource_group_name,
-                id=deployment.id,
-                outputs=deployment.properties.outputs,
-                instances=self._get_instances(deployment)
-            )
+            if deployment is None:
+                self.results['deployment'] = dict(
+                    name=self.deployment_name,
+                    group_name=self.resource_group_name,
+                    id=None,
+                    outputs=None,
+                    instances=None
+                )
+            else:
+                self.results['deployment'] = dict(
+                    name=deployment.name,
+                    group_name=self.resource_group_name,
+                    id=deployment.id,
+                    outputs=deployment.properties.outputs,
+                    instances=self._get_instances(deployment)
+                )
+
             self.results['changed'] = True
             self.results['msg'] = 'deployment succeeded'
         else:
-            if self.resource_group_exists(self.resource_group_name):
-                self.destroy_resource_group()
-                self.results['changed'] = True
-                self.results['msg'] = "deployment deleted"
+            try:
+                if self.get_resource_group(self.resource_group_name):
+                    self.destroy_resource_group()
+                    self.results['changed'] = True
+                    self.results['msg'] = "deployment deleted"
+            except CloudError:
+                # resource group does not exist
+                pass
 
         return self.results
 
@@ -478,21 +475,30 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
         :return:
         """
 
-        deploy_parameter = DeploymentProperties(self.deployment_mode)
+        deploy_parameter = self.rm_models.DeploymentProperties(self.deployment_mode)
         if not self.parameters_link:
             deploy_parameter.parameters = self.parameters
         else:
-            deploy_parameter.parameters_link = ParametersLink(
+            deploy_parameter.parameters_link = self.rm_models.ParametersLink(
                 uri=self.parameters_link
             )
         if not self.template_link:
             deploy_parameter.template = self.template
         else:
-            deploy_parameter.template_link = TemplateLink(
+            deploy_parameter.template_link = self.rm_models.TemplateLink(
                 uri=self.template_link
             )
 
-        params = ResourceGroup(location=self.location, tags=self.tags)
+        if self.append_tags and self.tags:
+            try:
+                rg = self.get_resource_group(self.resource_group_name)
+                if rg.tags:
+                    self.tags = dict(self.tags, **rg.tags)
+            except CloudError:
+                # resource group does not exist
+                pass
+
+        params = self.rm_models.ResourceGroup(location=self.location, tags=self.tags)
 
         try:
             self.rm_client.resource_groups.create_or_update(self.resource_group_name, params)
@@ -504,10 +510,11 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
                                                                  self.deployment_name,
                                                                  deploy_parameter)
 
-            deployment_result = self.get_poller_result(result)
+            deployment_result = None
             if self.wait_for_deployment_completion:
+                deployment_result = self.get_poller_result(result)
                 while deployment_result.properties is None or deployment_result.properties.provisioning_state not in ['Canceled', 'Failed', 'Deleted',
-                                                                              'Succeeded']:
+                                                                                                                      'Succeeded']:
                     time.sleep(self.wait_for_deployment_polling_period)
                     deployment_result = self.rm_client.deployments.get(self.resource_group_name, self.deployment_name)
         except CloudError as exc:
@@ -530,7 +537,7 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
         """
         try:
             result = self.rm_client.resource_groups.delete(self.resource_group_name)
-            result.wait() # Blocking wait till the delete is finished
+            result.wait()  # Blocking wait till the delete is finished
         except CloudError as e:
             if e.status_code == 404 or e.status_code == 204:
                 return
@@ -538,19 +545,6 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
                 self.fail("Delete resource group and deploy failed with status code: %s and message: %s" %
                           (e.status_code, e.message))
 
-    def resource_group_exists(self, resource_group):
-        '''
-        Return True/False based on existence of requested resource group.
-
-        :param resource_group: string. Name of a resource group.
-        :return: boolean
-        '''
-        try:
-            self.rm_client.resource_groups.get(resource_group)
-        except CloudError:
-            return False
-        return True
-
     def _get_failed_nested_operations(self, current_operations):
         new_operations = []
         for operation in current_operations:
@@ -564,7 +558,7 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
                                                                                       nested_deployment)
                     except CloudError as exc:
                         self.fail("List nested deployment operations failed with status code: %s and message: %s" %
-                                 (e.status_code, e.message))
+                                  (exc.status_code, exc.message))
                     new_nested_operations = self._get_failed_nested_operations(nested_operations)
                     new_operations += new_nested_operations
         return new_operations
@@ -622,7 +616,7 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
         for dep in dependencies:
             if dep.resource_name not in tree:
                 tree[dep.resource_name] = dict(dep=dep, children=dict())
-            if isinstance(dep, Dependency) and dep.depends_on is not None and len(dep.depends_on) > 0:
+            if isinstance(dep, self.rm_models.Dependency) and dep.depends_on is not None and len(dep.depends_on) > 0:
                 self._build_hierarchy(dep.depends_on, tree[dep.resource_name]['children'])
 
         if 'top' in tree:
@@ -637,30 +631,29 @@ class AzureRMDeploymentManager(AzureRMModuleBase):
 
     def _get_ip_dict(self, ip):
         ip_dict = dict(name=ip.name,
-            id=ip.id,
-            public_ip=ip.ip_address,
-            public_ip_allocation_method=str(ip.public_ip_allocation_method)
-        )
+                       id=ip.id,
+                       public_ip=ip.ip_address,
+                       public_ip_allocation_method=str(ip.public_ip_allocation_method)
+                       )
         if ip.dns_settings:
             ip_dict['dns_settings'] = {
-                'domain_name_label':ip.dns_settings.domain_name_label,
-                'fqdn':ip.dns_settings.fqdn
+                'domain_name_label': ip.dns_settings.domain_name_label,
+                'fqdn': ip.dns_settings.fqdn
             }
         return ip_dict
 
     def _nic_to_public_ips_instance(self, nics):
         return [self.network_client.public_ip_addresses.get(public_ip_id.split('/')[4], public_ip_id.split('/')[-1])
-                  for nic_obj in [self.network_client.network_interfaces.get(self.resource_group_name,
-                                                                             nic['dep'].resource_name) for nic in nics]
-                  for public_ip_id in [ip_conf_instance.public_ip_address.id
-                                       for ip_conf_instance in nic_obj.ip_configurations
-                                       if ip_conf_instance.public_ip_address]]
+                for nic_obj in (self.network_client.network_interfaces.get(self.resource_group_name,
+                                                                           nic['dep'].resource_name) for nic in nics)
+                for public_ip_id in [ip_conf_instance.public_ip_address.id
+                                     for ip_conf_instance in nic_obj.ip_configurations
+                                     if ip_conf_instance.public_ip_address]]
 
 
 def main():
     AzureRMDeploymentManager()
 
-from ansible.module_utils.basic import *
+
 if __name__ == '__main__':
     main()
-
