@@ -1,33 +1,20 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-# (c) 2012, Matt Wright <matt@nobien.net>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
-#
+# Copyright: (c) 2012, Matt Wright <matt@nobien.net>
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
-                    'supported_by': 'curated'}
-
+                    'supported_by': 'core'}
 
 DOCUMENTATION = '''
 ---
 module: pip
-short_description: Manages Python library dependencies.
+short_description: Manages Python library dependencies
 description:
      - "Manage Python library dependencies. To use this module, one of the following keys is required: C(name)
        or C(requirements)."
@@ -37,19 +24,13 @@ options:
     description:
       - The name of a Python library to install or the url of the remote package.
       - As of 2.2 you can supply a list of names.
-    required: false
-    default: null
   version:
     description:
-      - The version number to install of the Python library specified in the I(name) parameter
-    required: false
-    default: null
+      - The version number to install of the Python library specified in the I(name) parameter.
   requirements:
     description:
       - The path to a pip requirements file, which should be local to the remote system.
         File can be specified as a relative path if using the chdir option.
-    required: false
-    default: null
   virtualenv:
     description:
       - An optional path to a I(virtualenv) directory to install into.
@@ -58,62 +39,51 @@ options:
         If the virtualenv does not exist, it will be created before installing
         packages. The optional virtualenv_site_packages, virtualenv_command,
         and virtualenv_python options affect the creation of the virtualenv.
-    required: false
-    default: null
   virtualenv_site_packages:
-    version_added: "1.0"
     description:
       - Whether the virtual environment will inherit packages from the
         global site-packages directory.  Note that if this setting is
         changed on an already existing virtual environment it will not
         have any effect, the environment must be deleted and newly
         created.
-    required: false
+    type: bool
     default: "no"
-    choices: [ "yes", "no" ]
+    version_added: "1.0"
   virtualenv_command:
-    version_added: "1.1"
     description:
       - The command or a pathname to the command to create the virtual
         environment with. For example C(pyvenv), C(virtualenv),
         C(virtualenv2), C(~/bin/virtualenv), C(/usr/local/bin/virtualenv).
-    required: false
     default: virtualenv
+    version_added: "1.1"
   virtualenv_python:
-    version_added: "2.0"
     description:
       - The Python executable used for creating the virtual environment.
         For example C(python3.5), C(python2.7). When not specified, the
         Python version used to run the ansible module is used. This parameter
         should not be used when C(virtualenv_command) is using C(pyvenv) or
         the C(-m venv) module.
-    required: false
-    default: null
+    version_added: "2.0"
   state:
     description:
       - The state of module
       - The 'forcereinstall' option is only available in Ansible 2.1 and above.
-    required: false
+    choices: [ absent, forcereinstall, latest, present ]
     default: present
-    choices: [ "present", "absent", "latest", "forcereinstall" ]
   extra_args:
     description:
       - Extra arguments passed to pip.
-    required: false
-    default: null
     version_added: "1.0"
   editable:
     description:
-      - Pass the editable flag for versioning URLs.
-    required: false
-    default: yes
+      - Pass the editable flag.
+    type: bool
+    default: 'no'
     version_added: "2.0"
   chdir:
     description:
       - cd into this directory before running the command
     version_added: "1.3"
-    required: false
-    default: null
   executable:
     description:
       - The explicit executable or a pathname to the executable to be used to
@@ -124,8 +94,6 @@ options:
         By default, it will take the appropriate version for the python interpreter
         use by ansible, e.g. pip3 on python 3, and pip2 or pip on python 2.
     version_added: "1.3"
-    required: false
-    default: null
   umask:
     description:
       - The system umask to apply before installing the pip package. This is
@@ -134,17 +102,17 @@ options:
         packages which are to be used by all users. Note that this requires you
         to specify desired umask mode in octal, with a leading 0 (e.g., 0077).
     version_added: "2.1"
-    required: false
-    default: null
-
 notes:
    - Please note that virtualenv (U(http://www.virtualenv.org/)) must be
      installed on the remote host if the virtualenv parameter is specified and
      the virtualenv needs to be created.
    - By default, this module will use the appropriate version of pip for the
      interpreter used by ansible (e.g. pip3 when using python 3, pip2 otherwise)
-requirements: [ "virtualenv", "pip" ]
-author: "Matt Wright (@mattupstate)"
+requirements:
+- pip
+- virtualenv
+author:
+- Matt Wright (@mattupstate)
 '''
 
 EXAMPLES = '''
@@ -161,10 +129,9 @@ EXAMPLES = '''
 - pip:
     name: svn+http://myrepo/svn/MyApp#egg=MyApp
 
-# Install MyApp using one of the remote protocols (bzr+,hg+,git+) in a non editable way.
+# Install MyApp using one of the remote protocols (bzr+,hg+,git+).
 - pip:
     name: git+http://myrepo/app/MyApp
-    editable: false
 
 # Install (MyApp) from local tarball
 - pip:
@@ -187,6 +154,11 @@ EXAMPLES = '''
     virtualenv: /my_app/venv
     virtualenv_command: virtualenv-2.7
 
+# Install (Bottle) within a user home directory.
+- pip:
+    name: bottle
+    extra_args: --user
+
 # Install specified python requirements.
 - pip:
     requirements: /my_app/requirements.txt
@@ -218,15 +190,16 @@ EXAMPLES = '''
   become: True
 '''
 
-import tempfile
-import re
 import os
+import re
 import sys
+import tempfile
 
 from ansible.module_utils.basic import AnsibleModule, is_executable
 from ansible.module_utils._text import to_native
 from ansible.module_utils.six import PY3
 
+
 #: Python one-liners to be run at the command line that will determine the
 # installed version for these special libraries.  These are libraries that
 # don't end up in the output of pip freeze.
@@ -256,7 +229,7 @@ def _get_full_name(name, version=None):
 def _get_packages(module, pip, chdir):
     '''Return results of pip command to get packages.'''
     # Try 'pip list' command first.
-    command = '%s list' % pip
+    command = '%s list --format=freeze' % pip
     lang_env = dict(LANG='C', LC_ALL='C', LC_MESSAGES='C')
     rc, out, err = module.run_command(command, cwd=chdir, environ_update=lang_env)
 
@@ -273,19 +246,8 @@ def _get_packages(module, pip, chdir):
 def _is_present(name, version, installed_pkgs, pkg_command):
     '''Return whether or not package is installed.'''
     for pkg in installed_pkgs:
-        # Package listing will be different depending on which pip
-        # command was used ('pip list' vs. 'pip freeze').
-        if 'list' in pkg_command:
-            pkg = pkg.replace('(', '').replace(')', '')
-            if ',' in pkg:
-                pkg_name, pkg_version, _ = pkg.replace(',', '').split(' ')
-            else:
-                pkg_name, pkg_version = pkg.split(' ')
-        elif 'freeze' in pkg_command:
-            if '==' in pkg:
-                pkg_name, pkg_version = pkg.split('==')
-            else:
-                continue
+        if '==' in pkg:
+            pkg_name, pkg_version = pkg.split('==')
         else:
             continue
 
@@ -307,7 +269,6 @@ def _get_pip(module, env=None, executable=None):
 
     pip = None
     if executable is not None:
-        executable = os.path.expanduser(executable)
         if os.path.isabs(executable):
             pip = executable
         else:
@@ -326,7 +287,7 @@ def _get_pip(module, env=None, executable=None):
                 # For-else: Means that we did not break out of the loop
                 # (therefore, that pip was not found)
                 module.fail_json(msg='Unable to find any of %s to use.  pip'
-                        ' needs to be installed.' % ', '.join(candidate_pip_basenames))
+                                     ' needs to be installed.' % ', '.join(candidate_pip_basenames))
         else:
             # If we're using a virtualenv we must use the pip from the
             # virtualenv
@@ -340,10 +301,9 @@ def _get_pip(module, env=None, executable=None):
             else:
                 # For-else: Means that we did not break out of the loop
                 # (therefore, that pip was not found)
-                module.fail_json(msg='Unable to find pip in the virtualenv,'
-                        ' %s, under any of these names: %s. Make sure pip is'
-                        ' present in the virtualenv.' % (env,
-                            ', '.join(candidate_pip_basenames)))
+                module.fail_json(msg='Unable to find pip in the virtualenv, %s, ' % env +
+                                     'under any of these names: %s. ' % (', '.join(candidate_pip_basenames)) +
+                                     'Make sire pip is present in the virtualenv.')
 
     return pip
 
@@ -382,6 +342,53 @@ def _get_package_info(module, package, env=None):
     return formatted_dep
 
 
+def setup_virtualenv(module, env, chdir, out, err):
+    if module.check_mode:
+        module.exit_json(changed=True)
+
+    cmd = module.params['virtualenv_command']
+    if os.path.basename(cmd) == cmd:
+        cmd = module.get_bin_path(cmd, True)
+
+    if module.params['virtualenv_site_packages']:
+        cmd += ' --system-site-packages'
+    else:
+        cmd_opts = _get_cmd_options(module, cmd)
+        if '--no-site-packages' in cmd_opts:
+            cmd += ' --no-site-packages'
+
+    virtualenv_python = module.params['virtualenv_python']
+    # -p is a virtualenv option, not compatible with pyenv or venv
+    # this if validates if the command being used is not any of them
+    if not any(ex in module.params['virtualenv_command'] for ex in ('pyvenv', '-m venv')):
+        if virtualenv_python:
+            cmd += ' -p%s' % virtualenv_python
+        elif PY3:
+            # Ubuntu currently has a patch making virtualenv always
+            # try to use python2.  Since Ubuntu16 works without
+            # python2 installed, this is a problem.  This code mimics
+            # the upstream behaviour of using the python which invoked
+            # virtualenv to determine which python is used inside of
+            # the virtualenv (when none are specified).
+            cmd += ' -p%s' % sys.executable
+
+    # if venv or pyvenv are used and virtualenv_python is defined, then
+    # virtualenv_python is ignored, this has to be acknowledged
+    elif module.params['virtualenv_python']:
+        module.fail_json(
+            msg='virtualenv_python should not be used when'
+                ' using the venv module or pyvenv as virtualenv_command'
+        )
+
+    cmd = "%s %s" % (cmd, env)
+    rc, out_venv, err_venv = module.run_command(cmd, cwd=chdir)
+    out += out_venv
+    err += err_venv
+    if rc != 0:
+        _fail(module, cmd, out, err)
+    return out, err
+
+
 def main():
     state_map = dict(
         present='install',
@@ -392,24 +399,24 @@ def main():
 
     module = AnsibleModule(
         argument_spec=dict(
-            state=dict(default='present', choices=state_map.keys()),
+            state=dict(type='str', default='present', choices=state_map.keys()),
             name=dict(type='list'),
             version=dict(type='str'),
-            requirements=dict(),
+            requirements=dict(type='str'),
             virtualenv=dict(type='path'),
-            virtualenv_site_packages=dict(default=False, type='bool'),
-            virtualenv_command=dict(default='virtualenv', type='path'),
+            virtualenv_site_packages=dict(type='bool', default=False),
+            virtualenv_command=dict(type='path', default='virtualenv'),
             virtualenv_python=dict(type='str'),
-            use_mirrors=dict(default=True, type='bool'),
-            extra_args=dict(),
-            editable=dict(default=True, type='bool'),
+            use_mirrors=dict(type='bool', default=True),
+            extra_args=dict(type='str'),
+            editable=dict(type='bool', default=False),
             chdir=dict(type='path'),
-            executable=dict(),
-            umask=dict(),
+            executable=dict(type='path'),
+            umask=dict(type='str'),
         ),
         required_one_of=[['name', 'requirements']],
         mutually_exclusive=[['name', 'requirements'], ['executable', 'virtualenv']],
-        supports_check_mode=True
+        supports_check_mode=True,
     )
 
     state = module.params['state']
@@ -417,9 +424,13 @@ def main():
     version = module.params['version']
     requirements = module.params['requirements']
     extra_args = module.params['extra_args']
-    virtualenv_python = module.params['virtualenv_python']
     chdir = module.params['chdir']
     umask = module.params['umask']
+    env = module.params['virtualenv']
+
+    venv_created = False
+    if chdir:
+        env = os.path.join(chdir, env)
 
     if umask and not isinstance(umask, int):
         try:
@@ -442,52 +453,10 @@ def main():
         err = ''
         out = ''
 
-        env = module.params['virtualenv']
-
         if env:
             if not os.path.exists(os.path.join(env, 'bin', 'activate')):
-                if module.check_mode:
-                    module.exit_json(changed=True)
-
-                cmd = module.params['virtualenv_command']
-                if os.path.basename(cmd) == cmd:
-                    cmd = module.get_bin_path(cmd, True)
-
-                if module.params['virtualenv_site_packages']:
-                    cmd += ' --system-site-packages'
-                else:
-                    cmd_opts = _get_cmd_options(module, cmd)
-                    if '--no-site-packages' in cmd_opts:
-                        cmd += ' --no-site-packages'
-
-                # -p is a virtualenv option, not compatible with pyenv or venv
-                # this if validates if the command being used is not any of them
-                if not any(ex in module.params['virtualenv_command'] for ex in ('pyvenv', '-m venv')):
-                    if virtualenv_python:
-                        cmd += ' -p%s' % virtualenv_python
-                    elif PY3:
-                        # Ubuntu currently has a patch making virtualenv always
-                        # try to use python2.  Since Ubuntu16 works without
-                        # python2 installed, this is a problem.  This code mimics
-                        # the upstream behaviour of using the python which invoked
-                        # virtualenv to determine which python is used inside of
-                        # the virtualenv (when none are specified).
-                        cmd += ' -p%s' % sys.executable
-
-                # if venv or pyvenv are used and virtualenv_python is defined, then
-                # virtualenv_python is ignored, this has to be acknowledged
-                elif module.params['virtualenv_python']:
-                    module.fail_json(
-                        msg='virtualenv_python should not be used when'
-                            ' using the venv module or pyvenv as virtualenv_command'
-                    )
-
-                cmd = "%s %s" % (cmd, env)
-                rc, out_venv, err_venv = module.run_command(cmd, cwd=chdir)
-                out += out_venv
-                err += err_venv
-                if rc != 0:
-                    _fail(module, cmd, out, err)
+                venv_created = True
+                out, err = setup_virtualenv(module, env, chdir, out, err)
 
         pip = _get_pip(module, env, module.params['executable'])
 
@@ -512,7 +481,7 @@ def main():
                     has_vcs = True
                     break
 
-        if has_vcs and module.params['editable']:
+        if module.params['editable']:
             args_list = []  # used if extra_args is not used at all
             if extra_args:
                 args_list = extra_args.split(' ')
@@ -527,15 +496,17 @@ def main():
         if name:
             for pkg in name:
                 cmd += ' %s' % _get_full_name(pkg, version)
+        elif requirements:
+            cmd += ' -r %s' % requirements
         else:
-            if requirements:
-                cmd += ' -r %s' % requirements
+            module.exit_json(
+                changed=False,
+                warnings=["No valid name or requirements file found."],
+            )
 
         if module.check_mode:
             if extra_args or requirements or state == 'latest' or not name:
                 module.exit_json(changed=True)
-            elif has_vcs:
-                module.exit_json(changed=True)
 
             pkg_cmd, out_pip, err_pip = _get_packages(module, pip, chdir)
 
@@ -564,10 +535,9 @@ def main():
                         break
             module.exit_json(changed=changed, cmd=pkg_cmd, stdout=out, stderr=err)
 
+        out_freeze_before = None
         if requirements or has_vcs:
             _, out_freeze_before, _ = _get_packages(module, pip, chdir)
-        else:
-            out_freeze_before = None
 
         rc, out_pip, err_pip = module.run_command(cmd, path_prefix=path_prefix, cwd=chdir)
         out += out_pip
@@ -584,11 +554,10 @@ def main():
             if out_freeze_before is None:
                 changed = 'Successfully installed' in out_pip
             else:
-                if out_freeze_before is None:
-                    changed = 'Successfully installed' in out_pip
-                else:
-                    _, out_freeze_after, _ = _get_packages(module, pip, chdir)
-                    changed = out_freeze_before != out_freeze_after
+                _, out_freeze_after, _ = _get_packages(module, pip, chdir)
+                changed = out_freeze_before != out_freeze_after
+
+        changed = changed or venv_created
 
         module.exit_json(changed=changed, cmd=cmd, name=name, version=version,
                          state=state, requirements=requirements, virtualenv=env,
@@ -597,5 +566,6 @@ def main():
         if old_umask is not None:
             os.umask(old_umask)
 
+
 if __name__ == '__main__':
     main()
